##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################
"""Test setup for ZEO connection logic.

The actual tests are in ConnectionTests.py; this file provides the
platform-dependent scaffolding.
"""

import unittest

from zodb.zeo.tests.connection import ConnectionTests, ReconnectionTests
from zodb.storage.base import berkeley_is_available

class FileStorageConfig:
    def getConfig(self, path, create, read_only):
        return """\
        <Storage>
            type FileStorage
            file_name %s
            create %s
            read_only %s
        </Storage>""" % (path,
                         create and 'yes' or 'no',
                         read_only and 'yes' or 'no')

class BerkeleyStorageConfig:
    def getConfig(self, path, create, read_only):
        # Full always creates and doesn't have a read_only flag
        return """\
        <Storage>
            type BDBFullStorage
            name %s
            read_only %s
        </Storage>""" % (path,
                         read_only and 'yes' or 'no')

class MappingStorageConfig:
    def getConfig(self, path, create, read_only):
        return """\
        <Storage>
            type MappingStorage
            name %s
        </Storage>""" % path

tests = [
    (MappingStorageConfig, ConnectionTests, 1),
    (FileStorageConfig, ReconnectionTests, 1),
    (FileStorageConfig, ConnectionTests, 2),
    ]
         
if berkeley_is_available:
    tests += [
        (BerkeleyStorageConfig, ConnectionTests, 2),
        (BerkeleyStorageConfig, ReconnectionTests, 2),
        ]

def test_suite():
    suite = unittest.TestSuite()
    for testclass, configclass, level in tests:
        # synthesize a concrete class combining tests and configuration
        name = "%s:%s" % (testclass.__name__, configclass.__name__)
        aclass = type.__new__(type, name, (configclass, testclass, object), {})
        aclass.level = level
        sub = unittest.makeSuite(aclass, "check")
        suite.addTest(sub)
    return suite
